package com.diaoyun.zion.master.exception;

import com.alibaba.fastjson.JSON;
import com.diaoyun.zion.master.base.Result;
import com.diaoyun.zion.master.enums.ResultCodeEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.web.servlet.error.DefaultErrorAttributes;
import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

/**
 *
 */
@RestControllerAdvice
public class GlobalExceptionHandlerController {

  private final Logger logger = LoggerFactory.getLogger(getClass());
  @Bean
  public ErrorAttributes errorAttributes() {
    // Hide exception field in the return object
    return new DefaultErrorAttributes() {
      @Override
      public Map<String, Object> getErrorAttributes(WebRequest requestAttributes, boolean includeStackTrace) {
        Map<String, Object> errorAttributes = super.getErrorAttributes(requestAttributes, includeStackTrace);
        errorAttributes.remove("exception");
        errorAttributes.put("code", errorAttributes.get("status"));
        //处理403禁止访问
        if(403==(int)errorAttributes.get("status")) {
          errorAttributes.put("code", ResultCodeEnum.NEED_LOGIN.getCode());
          errorAttributes.put("message", ResultCodeEnum.NEED_LOGIN.getDesc());
        }

        return errorAttributes;
      }

      @Override
      public ModelAndView resolveException(HttpServletRequest request, HttpServletResponse response, Object handler,
                                           Exception exception) {
        Result result;
        if (exception instanceof ApplicationException) {
          ApplicationException ae = (ApplicationException) exception;
          result = new Result(ae.getCode(),ae.getMessage());
        }
        else {
          result = new Result(ResultCodeEnum.ERROR.getCode(),exception.getMessage());
          logger.error(exception.getMessage(), exception);
        }

        response.setContentType("application/json;charset=UTF-8");
        response.setStatus(HttpStatus.OK.value());
        PrintWriter writer=null;
        try {
          writer = response.getWriter();
          writer.write(JSON.toJSONString(result));

        }
        catch (IOException ie) {
          logger.error("Failed to serialize the object to json for exception resolver!", ie);
        } finally {
          writer.flush();
          writer.close();
        }
        return new ModelAndView();
      }
    };
  }

  /*@ExceptionHandler(ApplicationException.class)
  public void handleCustomException(HttpServletResponse res, ApplicationException ex) throws IOException {
    res.sendError(ex.getCode(), ex.getMessage());
  }

  @ExceptionHandler(AccessDeniedException.class)
  public void handleAccessDeniedException(HttpServletResponse res,Exception ex) throws IOException {
    res.sendError(HttpStatus.FORBIDDEN.value(), ex.getMessage());
  }

  @ExceptionHandler(Exception.class)
  @ResponseStatus(HttpStatus.FORBIDDEN)
  public void handleException(HttpServletResponse res,Exception ex) throws IOException {
    res.sendError(HttpStatus.BAD_REQUEST.value(), ex.getMessage());
  }*/




}
